package com.bomostory.sceneeditmodule.screen.movie

import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.ViewModel
import android.content.ContentResolver
import android.media.MediaMetadataRetriever
import com.bomostory.sceneeditmodule.basicdata.*
import com.bomostory.sceneeditmodule.utils.TimeUtils
import com.example.exportmedia.MediaHelper
import com.example.exportmedia.audio.AudioEditor
import io.reactivex.Observable
import java.io.File

class MovieEditViewModel : ViewModel() {

    val storyLiveData = MutableLiveData<Story>()

    val storyPeriodLiveData = MutableLiveData<Long>()

    val sceneLiveData = MutableLiveData<Scene>()

    val musicLiveData = MutableLiveData<Music>()

    val musicListLiveData = MutableLiveData<List<Music>>().apply {
        value = ArrayList()
    }

    lateinit var mediaHelper: MediaHelper

    var project: Project? = null
        set(value) {
            field = value
            value?.let {
                musicSource.library = parseThemeMusic(it)

                it.story?.let {
                    storyLiveData.value = it
                    storyPeriodLiveData.value = parseStoryPeriod(it)
                    sceneLiveData.value = it.scenes[0]
                }
            }
        }

    var musicSource = MusicSource()


    fun parseDeviceMusic(contentResolver: ContentResolver) {
        val libMusicPaths = ArrayList<String?>()
        for (libMusic in musicSource.library) {
            libMusicPaths.add(libMusic.path)
        }

        musicSource.device = DataParser.parseMusic(contentResolver, libMusicPaths)
    }

    fun exportEditAudio(music: Music, outputFile: File): Observable<String> {
        if (music.endTime - music.startTime > storyPeriodLiveData.value ?: 0) {
            music.endTime = music.startTime + (storyPeriodLiveData.value ?: 0)
        }

        val audioEditorBuilder = AudioEditor.Builder(mediaHelper)
        audioEditorBuilder.editAudioFile = File(music.path)
        audioEditorBuilder.startTime = TimeUtils.getEncodeTimeFormt(music.startTime)
        audioEditorBuilder.endTime = TimeUtils.getEncodeTimeFormt(music.endTime)
        audioEditorBuilder.volume = music.volume

        val audioEditor = audioEditorBuilder.build()
        return audioEditor.output(outputFile)
    }

    private fun parseThemeMusic(project: Project): ArrayList<Music> {
        val musics = ArrayList<Music>()
        val mediaMetadataRetriever = MediaMetadataRetriever()
        val assetFolder = project.assetFolder

        project.themeAssetIndex.contains.music.values.forEach {
            val musicPath = "$assetFolder/$it"
            val file = File(musicPath)

            mediaMetadataRetriever.setDataSource(musicPath)

            val music = Music()
            music.name = file.name
            music.path = file.path
            music.duration = mediaMetadataRetriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION).toLong()
            music.endTime = mediaMetadataRetriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION).toLong()
            musics.add(music)
        }
        return musics
    }

    private fun parseStoryPeriod(story: Story): Long {
        var totalPeriod: Long = 0
        for (scene in story.scenes) {
            val period = scene.record?.period ?: 0
            totalPeriod += period
        }
        return totalPeriod
    }

}